/****************************************************************************
*                   nexus.h
*
*  This module contains all defines, typedefs, and prototypes for OVUS.C.
*
*  This module was written by Jerome Grimbert. 
*
*  from Persistence of Vision(tm) Ray Tracer
*  Copyright 2000 J. Grimbert
*---------------------------------------------------------------------------
*  NOTICE: This source code file is provided so that users may experiment
*  with enhancements to POV-Ray and to port the software to platforms other
*  than those supported by the POV-Ray Team.  There are strict rules under
*  which you are permitted to use this file.  The rules are in the file
*  named POVLEGAL.DOC which should be distributed with this file.
*  If POVLEGAL.DOC is not available or for more info please contact the POV-Ray
*  Team Coordinator by email to team-coord@povray.org or visit us on the web at
*  http://www.povray.org. The latest version of POV-Ray may be found at this site.
*
* This program is based on the popular DKB raytracer version 2.12.
* DKBTrace was originally written by David K. Buck.
* DKBTrace Ver 2.0-2.12 were written by David K. Buck & Aaron A. Collins.
*
*****************************************************************************/


#ifndef OVUS_H
#define OVUS_H



/*****************************************************************************
* Global preprocessor defines
******************************************************************************/

#define OVUS_OBJECT (STURM_OK_OBJECT)


/*****************************************************************************
* Global typedefs
******************************************************************************/

typedef struct Ovus_Struct OVUS;
/*
 * Ovus structure
 * 
 * br: bottom radius (provided)
 * tr: top radius (provided)
 * xp: horizontal position of center of connecting surface (computed)
 * yp: vertical position of center of connecting surface (computed)
 * by: lowest vertical for the connection surface (computed)
 * ty: highest vertical for the connection surface (computed)
 * cr: radius of the connection surface (computed)
 * 
*/

struct Ovus_Struct
{
  OBJECT_FIELDS
  TRANSFORM *Trans;
  DBL br,tr,xp,yp,by,ty,cr;
};



/*****************************************************************************
* Global variables
******************************************************************************/

extern METHODS Ovus_Methods;



/*****************************************************************************
* Global functions
******************************************************************************/

OVUS *Create_OVUS (void);
void Compute_OVUS_BBox (OBJECT *Object);



#endif
